@@ -41,14 +41,16 @@ def main():
     converter = _get_converter()
     try:
         converter()
-    except (IOError, uxf.Error) as err:
+    except (OSError, uxf.Error) as err:
         print(f'uxfconvert:error:{err}', file=sys.stderr)
 
 
 def _get_converter():
     parser = argparse.ArgumentParser(usage=PREFIX + _get_usage())
-    parser.add_argument('-i', '--indent', type=int, default=2,
-                        help='default: 2, range 0-8')
+    parser.add_argument('-d', '--dropunused', action='store_true',
+                        help='drop unused imports and ttypes')
+    parser.add_argument('-r', '--replaceimports', action='store_true',
+                        help='replace imports with their used ttypes')
     parser.add_argument('-v', '--verbose', action='store_true')
     parser.add_argument(
         '-f', '--fieldnames', action='store_true',
@@ -71,8 +73,6 @@ def _get_usage():
 
 
 def _prepare_converter(parser, config):
-    if not (0 <= config.indent <= 8):
-        config.indent = 2 # sanitize rather than complain
     if len(config.file) < 2:
         parser.error('least two filenames are required')
     config.convert = None
@@ -105,42 +105,40 @@ def _get_other_converter(parser, config):
             uoutfile.endswith((DOT_UXF, DOT_UXF_GZ))):
         parser.error('cannot convert uxf to uxf, instead use: '
                      'python3 -m uxf infile.uxf outfile.uxf')
-    if uoutfile.endswith((DOT_UXF, DOT_UXF_GZ)):
+    kwargs = dict(verbose=config.verbose, drop_unused=config.dropunused,
+                  replace_imports=config.replaceimports)
+    if not uoutfile.endswith((DOT_CSV, DOT_INI, DOT_JSN, DOT_JSON,
+                              DOT_SQLITE, DOT_XML)):
         if uinfile.endswith(DOT_CSV):
             return lambda: csv_to_uxf(infile, outfile,
                                       fieldnames=config.fieldnames,
-                                      verbose=config.verbose)
+                                      **kwargs)
         elif uinfile.endswith(DOT_INI):
-            return lambda: ini_to_uxf(infile, outfile,
-                                      verbose=config.verbose)
+            return lambda: ini_to_uxf(infile, outfile, **kwargs)
         elif uinfile.endswith((DOT_JSN, DOT_JSON)):
-            return lambda: json_to_uxf(infile, outfile,
-                                       verbose=config.verbose)
+            return lambda: json_to_uxf(infile, outfile, **kwargs)
         elif uinfile.endswith(DOT_SQLITE):
-            return lambda: sqlite_to_uxf(infile, outfile,
-                                         verbose=config.verbose)
+            return lambda: sqlite_to_uxf(infile, outfile, **kwargs)
         elif uinfile.endswith(DOT_XML):
-            return lambda: xml_to_uxf(infile, outfile,
-                                      verbose=config.verbose)
-    elif uinfile.endswith((DOT_UXF, DOT_UXF_GZ)):
+            return lambda: xml_to_uxf(infile, outfile, **kwargs)
+    elif not uinfile.endswith((DOT_CSV, DOT_INI, DOT_JSN, DOT_JSON,
+                               DOT_SQLITE, DOT_XML)):
         if uoutfile.endswith(DOT_CSV):
-            return lambda: uxf_to_csv(infile, outfile,
-                                      verbose=config.verbose)
+            return lambda: uxf_to_csv(infile, outfile, **kwargs)
         elif uoutfile.endswith((DOT_JSN, DOT_JSON)):
-            return lambda: uxf_to_json(infile, outfile,
-                                       verbose=config.verbose)
+            return lambda: uxf_to_json(infile, outfile, **kwargs)
         elif uoutfile.endswith(DOT_SQLITE):
-            return lambda: uxf_to_sqlite(infile, outfile,
-                                         verbose=config.verbose)
+            return lambda: uxf_to_sqlite(infile, outfile, **kwargs)
         elif uoutfile.endswith(DOT_XML):
-            return lambda: uxf_to_xml(infile, outfile,
-                                      verbose=config.verbose)
+            return lambda: uxf_to_xml(infile, outfile, **kwargs)
     parser.error(f'cannot convert {infile} to {outfile}')
 
 
-def uxf_to_csv(infile, outfile, *, verbose=True):
+def uxf_to_csv(infile, outfile, *, verbose=True, replace_imports=False,
+               drop_unused=False):
     on_error = functools.partial(uxf.on_error, verbose=verbose)
-    uxo = uxf.load(infile, on_error=on_error)
+    uxo = uxf.load(infile, on_error=on_error, drop_unused=drop_unused,
+                   replace_imports=replace_imports)
     data = uxo.data
     if isinstance(data, uxf.Table):
         with open(outfile, 'w') as file:
@@ -161,60 +159,63 @@ def uxf_to_csv(infile, outfile, *, verbose=True):
                          'or a single list of lists of scalars to csv')
 
 
-def csv_to_uxf(infile, outfile, *, fieldnames=False, verbose=True):
+def csv_to_uxf(infile, outfile, *, fieldnames=False, verbose=True,
+               replace_imports=False, drop_unused=False):
     on_error = functools.partial(uxf.on_error, verbose=verbose)
-    data, filename, ttypes = _read_csv_to_data(infile, fieldnames)
-    uxf.dump(outfile, uxf.Uxf(data, custom=filename, ttypes=ttypes),
+    data, filename, tclasses = _read_csv_to_data(infile, fieldnames)
+    uxf.dump(outfile, uxf.Uxf(data, custom=filename, tclasses=tclasses),
              on_error=on_error)
 
 
 def _read_csv_to_data(infile, fieldnames):
-    ttypes = {}
+    tclasses = {}
     data = None
     with open(infile) as file:
         reader = csv.reader(file)
         for row in reader:
             if data is None:
                 if fieldnames:
-                    name = uxf.canonicalize(pathlib.Path(infile).stem)
-                    data = uxf.Table(name=name, fields=[uxf.Field(name)
-                                     for name in row])
-                    ttypes[name] = data.ttype
+                    ttype = uxf.canonicalize(pathlib.Path(infile).stem)
+                    data = uxf.table(ttype,
+                                     [uxf.Field(name) for name in row])
+                    tclasses[ttype] = data.tclass
                     continue
                 else:
                     data = []
-            row = [uxf.naturalize(x) for x in row]
-            if isinstance(data, uxf.Table):
-                data += row
-            else:
-                data.append(row)
-    return data, infile, ttypes
+            data.append([uxf.naturalize(x) for x in row])
+    return data, infile, tclasses
 
 
-def multi_csv_to_uxf(infiles, outfile, *, fieldnames=False, verbose=True):
+def multi_csv_to_uxf(infiles, outfile, *, fieldnames=False, verbose=True,
+                     replace_imports=False, drop_unused=False):
     data = []
-    ttypes = {}
+    tclasses = {}
     for infile in infiles:
-        datum, _, new_ttypes = _read_csv_to_data(infile, fieldnames)
+        datum, _, new_tclasses = _read_csv_to_data(infile, fieldnames)
         data.append(datum)
-        if new_ttypes:
-            ttypes.update(new_ttypes)
+        if new_tclasses:
+            tclasses.update(new_tclasses)
     on_error = functools.partial(uxf.on_error, verbose=verbose)
     uxf.dump(outfile,
-             uxf.Uxf(data, custom=' '.join(infiles), ttypes=ttypes),
+             uxf.Uxf(data, custom=' '.join(infiles), tclasses=tclasses),
              on_error=on_error)
 
 
-def uxf_to_json(infile, outfile, *, verbose=True):
+def uxf_to_json(infile, outfile, *, verbose=True, replace_imports=False,
+                drop_unused=False):
     on_error = functools.partial(uxf.on_error, verbose=verbose)
-    uxo = uxf.load(infile, on_error=on_error)
+    uxo = uxf.load(infile, on_error=on_error, drop_unused=drop_unused,
+                   replace_imports=replace_imports)
     d = {}
     if uxo.custom is not None:
         d[JSON_CUSTOM] = uxo.custom
     if uxo.comment is not None:
         d[JSON_COMMENT] = uxo.comment
-    if uxo.ttypes: # sorted helps round-trip regression tests
-        d[JSON_TTYPES] = sorted(uxo.ttypes.values(), key=lambda t: t.name)
+    if uxo.imports:
+        d[JSON_IMPORTS] = list(uxo.import_filenames)
+    if uxo.tclasses: # sorted helps round-trip regression tests
+        d[JSON_TCLASSES] = sorted(uxo.tclasses.values(),
+                                  key=lambda t: t.ttype)
     d[JSON_DATA] = uxo.data
     with open(outfile, 'wt', encoding=UTF8) as file:
         json.dump(d, file, cls=_JsonEncoder, indent=2)
@@ -242,15 +243,17 @@ class _JsonEncoder(json.JSONEncoder):
             return obj
         if isinstance(obj, uxf.Table):
             return {JSON_TABLE: dict(
-                name=obj.name, comment=obj.comment,
+                name=obj.ttype, comment=obj.comment,
                 fields={field.name: field.vtype for field in obj.fields},
                 records=obj.records)}
-        if isinstance(obj, uxf.TType):
-            d = dict(name=obj.name, fields={field.name: field.vtype
-                                            for field in obj.fields})
+        if isinstance(obj, uxf.TClass):
+            d = dict(name=obj.ttype)
+            if obj.fields:
+                d['fields'] = {field.name: field.vtype
+                               for field in obj.fields}
             if obj.comment is not None:
                 d[COMMENT] = obj.comment
-            return {JSON_TTYPE: d}
+            return {JSON_TCLASS: d}
         return json.JSONEncoder.default(self, obj)
 
 
@@ -285,21 +288,25 @@ def _json_encode_map(obj):
     return {JSON_MAP: m}
 
 
-def json_to_uxf(infile, outfile, *, verbose=True):
+def json_to_uxf(infile, outfile, *, verbose=True, replace_imports=False,
+                drop_unused=False):
     with open(infile, 'rt', encoding=UTF8) as file:
         d = json.load(file, object_hook=_json_naturalize)
     custom = d.get(JSON_CUSTOM)
     comment = d.get(JSON_COMMENT)
-    ttypes = None
-    ttype_list = d.get(JSON_TTYPES)
-    if ttype_list:
-        ttypes = {}
-        for ttype in ttype_list:
-            ttypes[ttype.name] = ttype
+    imports_list = d.get(JSON_IMPORTS)
+    imports = None if imports_list is None else _get_imports(imports_list)
+    tclasses = None
+    tclass_list = d.get(JSON_TCLASSES)
+    if tclass_list:
+        tclasses = {}
+        for tclass in tclass_list:
+            tclasses[tclass.ttype] = tclass
     data = d.get(JSON_DATA)
     on_error = functools.partial(uxf.on_error, verbose=verbose)
-    uxf.dump(outfile, uxf.Uxf(data, custom=custom, ttypes=ttypes,
-                              comment=comment), on_error=on_error)
+    uxo = uxf.Uxf(data, custom=custom, tclasses=tclasses, comment=comment)
+    uxo.imports = imports
+    uxf.dump(outfile, uxo, on_error=on_error)
 
 
 def _json_naturalize(d):
@@ -311,18 +318,14 @@ def _json_naturalize(d):
         return bytes.fromhex(d[JSON_BYTES])
     if JSON_LIST in d:
         jlist = d[JSON_LIST]
-        ls = uxf.List(jlist[LIST])
-        ls.comment = jlist[COMMENT]
-        ls.vtype = jlist[VTYPE]
-        return ls
+        return uxf.List(jlist[LIST], vtype=jlist[VTYPE],
+                        comment=jlist[COMMENT])
     if JSON_MAP in d:
         jmap = d[JSON_MAP]
         itype = jmap.get(ITYPE) # str or None
         itypes = jmap.get(ITYPES) # dict or None
-        m = uxf.Map()
-        m.comment = jmap[COMMENT]
-        m.ktype = jmap[KTYPE] # str or None
-        m.vtype = jmap[VTYPE] # str or None
+        m = uxf.Map(ktype=jmap[KTYPE], vtype=jmap[VTYPE],
+                    comment=jmap[COMMENT])
         for key, value in jmap[MAP].items():
             if itypes is not None:
                 itype = itypes.get(key)
@@ -336,17 +339,20 @@ def _json_naturalize(d):
         jtable = d[JSON_TABLE]
         fields = [uxf.Field(name, vtype) for name, vtype in
                   jtable[FIELDS].items()]
-        return uxf.Table(name=jtable[NAME], comment=jtable[COMMENT],
-                         fields=fields, records=jtable[RECORDS])
-    if JSON_TTYPE in d:
-        fields = [uxf.Field(name, vtype) for name, vtype in
-                  d[JSON_TTYPE][FIELDS].items()]
-        return uxf.TType(d[JSON_TTYPE][NAME], fields,
-                         comment=d[JSON_TTYPE].get(COMMENT))
+        return uxf.Table(uxf.TClass(jtable[NAME], fields),
+                         comment=jtable[COMMENT], records=jtable[RECORDS])
+    if JSON_TCLASS in d:
+        fields = d[JSON_TCLASS].get(FIELDS)
+        if fields is not None:
+            fields = [uxf.Field(name, vtype)
+                      for name, vtype in fields.items()]
+        return uxf.TClass(d[JSON_TCLASS][NAME], fields,
+                          comment=d[JSON_TCLASS].get(COMMENT))
     return d
 
 
-def ini_to_uxf(infile, outfile, *, verbose=True):
+def ini_to_uxf(infile, outfile, *, verbose=True, replace_imports=False,
+               drop_unused=False):
     on_error = functools.partial(uxf.on_error, verbose=verbose)
     ini = configparser.ConfigParser()
     ini.read(infile)
@@ -360,9 +366,11 @@ def ini_to_uxf(infile, outfile, *, verbose=True):
     uxf.dump(outfile, uxf.Uxf(data, custom=infile), on_error=on_error)
 
 
-def uxf_to_sqlite(infile, outfile, *, verbose=True):
+def uxf_to_sqlite(infile, outfile, *, verbose=True, replace_imports=False,
+                  drop_unused=False):
     on_error = functools.partial(uxf.on_error, verbose=verbose)
-    uxo = uxf.load(infile, on_error=on_error)
+    uxo = uxf.load(infile, on_error=on_error, drop_unused=drop_unused,
+                   replace_imports=replace_imports)
     if isinstance(uxo.data, uxf.Table):
         _uxf_to_sqlite(outfile, [uxo.data])
     elif (isinstance(uxo.data, (list, uxf.List)) and uxo.data and
@@ -381,8 +389,8 @@ def _uxf_to_sqlite(filename, tables):
     try:
         db = sqlite3.connect(filename)
         for table_index, table in enumerate(tables, 1):
-            table_name = _create_table(db, table, table_index)
-            _populate_table(db, table, table_name)
+            ttype = _create_table(db, table, table_index)
+            _populate_table(db, table, ttype)
     finally:
         if db is not None:
             db.commit()
@@ -390,8 +398,8 @@ def _uxf_to_sqlite(filename, tables):
 
 
 def _create_table(db, table, table_index):
-    table_name = uxf.canonicalize(table.name)
-    sql = ['CREATE TABLE IF NOT EXISTS ', table_name, ' (']
+    ttype = uxf.canonicalize(table.ttype)
+    sql = ['CREATE TABLE IF NOT EXISTS ', ttype, ' (']
     types = ['TEXT'] * len(table.fields)
     if table.records:
         for i, value in enumerate(table.records[0]):
@@ -411,17 +419,18 @@ def _create_table(db, table, table_index):
     sql += [');']
     cursor = db.cursor()
     cursor.execute(''.join(sql))
-    return table_name
+    return ttype
 
 
-def _populate_table(db, table, table_name):
-    sql = ['INSERT INTO ', table_name, ' VALUES (',
+def _populate_table(db, table, ttype):
+    sql = ['INSERT INTO ', ttype, ' VALUES (',
            ', '.join('?' * len(table.fields)), ');']
     cursor = db.cursor()
     cursor.executemany(''.join(sql), table.records)
 
 
-def sqlite_to_uxf(infile, outfile, *, verbose=True):
+def sqlite_to_uxf(infile, outfile, *, verbose=True, replace_imports=False,
+                  drop_unused=False):
     uxo = _sqlite_to_uxf(infile)
     on_error = functools.partial(uxf.on_error, verbose=verbose)
     uxo.dump(outfile, on_error=on_error)
@@ -432,27 +441,27 @@ def _sqlite_to_uxf(infile):
     try:
         db = sqlite3.connect(infile)
         uxo = uxf.Uxf()
-        table_names = []
+        ttypes = []
         comments = []
         cursor = db.cursor()
         sql = ("SELECT tbl_name, sql FROM sqlite_master "
                "WHERE type = 'table';")
-        for table_name, comment in cursor.execute(sql):
-            table_names.append(table_name)
+        for ttype, comment in cursor.execute(sql):
+            ttypes.append(ttype)
             comments.append(comment)
-        for table_name, comment in zip(table_names, comments):
+        for ttype, comment in zip(ttypes, comments):
             fields = []
             sql = 'SELECT name FROM pragma_table_info(?)'
-            for row in cursor.execute(sql, (table_name,)):
+            for row in cursor.execute(sql, (ttype,)):
                 fields.append(row[0])
-            table = uxf.Table(name=table_name, fields=fields,
-                              comment=comment)
+            table = uxf.table(ttype, fields, comment=comment)
             fields = [f'[{field}]' for field in fields]
-            sql = f'SELECT {", ".join(fields)} FROM {table_name};'
+            sql = f'SELECT {", ".join(fields)} FROM {ttype};'
             for row in cursor.execute(sql):
-                table += [uxf.naturalize(value) if isinstance(value, str)
-                          else value for value in row]
-            uxo.ttypes[table.ttype.name] = table.ttype
+                table.append(
+                    [uxf.naturalize(value) if isinstance(value, str) else
+                     value for value in row])
+            uxo.tclasses[table.ttype] = table.tclass
             uxo.data.append(table)
         return uxo
     finally:
@@ -460,9 +469,11 @@ def _sqlite_to_uxf(infile):
             db.close()
 
 
-def uxf_to_xml(infile, outfile, *, verbose=True):
+def uxf_to_xml(infile, outfile, *, verbose=True, replace_imports=False,
+               drop_unused=False):
     on_error = functools.partial(uxf.on_error, verbose=verbose)
-    uxo = uxf.load(infile, on_error=on_error)
+    uxo = uxf.load(infile, on_error=on_error, drop_unused=drop_unused,
+                   replace_imports=replace_imports)
     _uxf_to_xml(uxo, outfile)
 
 
@@ -475,33 +486,43 @@ def _uxf_to_xml(uxo, outfile):
         root.setAttribute('custom', uxo.custom)
     if uxo.comment:
         root.setAttribute('comment', uxo.comment)
-    if uxo.ttypes:
-        _xml_add_ttypes(tree, root, uxo.ttypes)
+    if uxo.imports:
+        _xml_add_imports(tree, root, uxo.import_filenames)
+    if uxo.tclasses:
+        _xml_add_tclasses(tree, root, uxo.tclasses)
     _xml_add_value(tree, root, uxo.data)
     with open(outfile, 'wt', encoding='utf-8') as file:
         file.write(tree.toprettyxml(indent='  '))
 
 
-def _xml_add_ttypes(tree, root, ttypes):
-    ttypes_element = tree.createElement('ttypes')
-    for ttype in sorted(ttypes.values()):
-        ttype_element = tree.createElement('ttype')
-        ttype_element.setAttribute('name', ttype.name)
-        if ttype.comment:
-            ttype_element.setAttribute('comment', ttype.comment)
-        for field in ttype.fields:
+def _xml_add_imports(tree, root, import_filenames):
+    imports_element = tree.createElement('imports')
+    for filename in import_filenames: # don't sort!
+        import_element = tree.createElement('import')
+        import_element.setAttribute('filename', filename)
+        imports_element.appendChild(import_element)
+    root.appendChild(imports_element)
+
+
+def _xml_add_tclasses(tree, root, tclasses):
+    tclasses_element = tree.createElement('ttypes')
+    for tclass in sorted(tclasses.values()):
+        tclass_element = tree.createElement('ttype')
+        tclass_element.setAttribute('name', tclass.ttype)
+        if tclass.comment:
+            tclass_element.setAttribute('comment', tclass.comment)
+        for field in tclass.fields:
             field_element = tree.createElement('field')
             field_element.setAttribute('name', field.name)
             if field.vtype is not None:
                 field_element.setAttribute('vtype', field.vtype)
-            ttype_element.appendChild(field_element)
-        ttypes_element.appendChild(ttype_element)
-    root.appendChild(ttypes_element)
+            tclass_element.appendChild(field_element)
+        tclasses_element.appendChild(tclass_element)
+    root.appendChild(tclasses_element)
 
 
 def _xml_add_value(tree, root, value):
-    if (isinstance(value, tuple) and
-            value.__class__.__name__.startswith('UXF')):
+    if value.__class__.__name__.startswith('UXF_'):
         _xml_add_list(tree, root, value, tag='row')
     elif isinstance(value, (set, frozenset, tuple, collections.deque, list,
                             uxf.List)):
@@ -550,7 +571,7 @@ def _xml_add_map(tree, root, map):
 
 def _xml_add_table(tree, root, table):
     table_element = tree.createElement('table')
-    table_element.setAttribute('name', table.name)
+    table_element.setAttribute('name', table.ttype)
     if table.comment is not None:
         table_element.setAttribute('comment', table.comment)
     for value in table:
@@ -593,7 +614,8 @@ def _xml_add_scalar(tree, root, value):
         raise SystemExit(f'invalid value type: {value} of {type(value)}')
 
 
-def xml_to_uxf(infile, outfile, *, verbose=True):
+def xml_to_uxf(infile, outfile, *, verbose=True, replace_imports=False,
+               drop_unused=False):
     on_error = functools.partial(uxf.on_error, verbose=verbose)
     uxo = _xml_to_uxf(infile)
     uxo.dump(outfile, on_error=on_error)
@@ -612,7 +634,8 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
     def __init__(self):
         super().__init__()
         self.stack = None
-        self.ttype = None
+        self.tclass = None
+        self.imports_list = []
         self.instr = False
         self.inbytes = False
         self.string = ''
@@ -625,26 +648,26 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
         if name == 'uxf':
             self.uxo.custom = d.get('custom', '')
             self.uxo.comment = d.get('comment')
-        elif name == 'ttypes':
+        elif name in {'imports', 'ttypes'}:
             pass
+        elif name == 'import':
+            self.imports_list.append(d['filename'])
         elif name == 'ttype':
-            self.ttype = uxf.TType(d['name'], comment=d.get('comment'))
+            self.tclass = uxf.TClass(d['name'], comment=d.get('comment'))
         elif name == 'field':
-            self.ttype.append(d['name'], d.get('vtype'))
+            self.tclass.append(d['name'], d.get('vtype'))
         elif name == 'map':
-            container = uxf.Map()
-            container.comment = d.get('comment')
-            container.ktype = d.get('ktype')
-            container.vtype = d.get('vtype')
+            container = uxf.Map(ktype=d.get('ktype'), vtype=d.get('vtype'),
+                                comment=d.get('comment'))
             self.start_container(container)
         elif name == 'list':
-            container = uxf.List()
-            container.comment = d.get('comment')
-            container.vtype = d.get('vtype')
+            container = uxf.List(vtype=d.get('vtype'),
+                                 comment=d.get('comment'))
             self.start_container(container)
         elif name == 'table':
-            container = uxf.Table(name=d['name'], comment=d.get('comment'))
-            container.ttype = self.uxo.ttypes[d['name']]
+            container = uxf.Table(uxf.TClass(d['name']),
+                                  comment=d.get('comment'))
+            container.tclass = self.uxo.tclasses[d['name']]
             self.start_container(container)
         elif name in {'key', 'value'}:
             pass # container.append() doesn't need this distinction
@@ -655,36 +678,39 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
             self.inbytes = True
             self.bytes = ''
         elif name == 'int':
-            self.stack[-1].append(int(d['v']))
+            _append_to_parent(self.stack, int(d['v']))
         elif name == 'real':
-            self.stack[-1].append(float(d['v']))
+            _append_to_parent(self.stack, float(d['v']))
         elif name == 'date':
-            self.stack[-1].append(uxf.naturalize(d['v']))
+            _append_to_parent(self.stack, uxf.naturalize(d['v']))
         elif name == 'datetime':
-            self.stack[-1].append(uxf.naturalize(d['v']))
+            _append_to_parent(self.stack, uxf.naturalize(d['v']))
         elif name == 'null':
-            self.stack[-1].append(None)
+            _append_to_parent(self.stack, None)
         elif name == 'yes':
-            self.stack[-1].append(True)
+            _append_to_parent(self.stack, True)
         elif name == 'no':
-            self.stack[-1].append(False)
+            _append_to_parent(self.stack, False)
 
 
     def endElement(self, name):
         if name in {'uxf', 'ttypes', 'field', 'key', 'value', 'int', 'real',
-                    'date', 'datetime', 'null', 'yes', 'no'}:
+                    'date', 'datetime', 'null', 'yes', 'no', 'import'}:
             pass
+        elif name == 'imports': # got them all (if any)
+            if self.imports_list:
+                self.uxo.imports = _get_imports(self.imports_list)
         elif name == 'ttype':
-            self.uxo.ttypes[self.ttype.name] = self.ttype
-            self.ttype = None
+            self.uxo.tclasses[self.tclass.ttype] = self.tclass
+            self.tclass = None
         elif name in {'map', 'list', 'table'}:
             self.stack.pop()
         elif name == 'str':
-            self.stack[-1].append(self.string)
+            _append_to_parent(self.stack, self.string)
             self.string = ''
             self.instr = False
         elif name == 'bytes':
-            self.stack[-1].append(bytes.fromhex(self.bytes))
+            _append_to_parent(self.stack, bytes.fromhex(self.bytes))
             self.bytes = ''
             self.inbytes = False
 
@@ -701,10 +727,27 @@ class _UxfSaxHandler(xml.sax.handler.ContentHandler):
             self.uxo.data = container
             self.stack = [container]
         elif self.stack:
-            self.stack[-1].append(container)
+            _append_to_parent(self.stack, container)
         self.stack.append(container)
 
 
+def _get_imports(imports_list):
+    imports = None
+    if imports_list:
+        uxt = ['uxf 1.0\n']
+        for import_filename in imports_list:
+            uxt.append(f'!{import_filename}\n')
+        uxt.append('[]\n')
+        uxt = ''.join(uxt)
+        uxo = uxf.loads(uxt)
+        imports = uxo.imports
+    return imports
+
+
+def _append_to_parent(stack, value):
+    uxf.append_to_parent(stack[-1], value)
+
+
 BYTES = 'bytes'
 COMMENT = 'comment'
 DOT_CSV = '.CSV'
@@ -720,8 +763,9 @@ ITYPE = 'itype'
 ITYPES = 'itypes'
 JSON_CUSTOM = 'UXF^custom'
 JSON_COMMENT = 'UXF^comment'
-JSON_TTYPES = 'UXF^ttypes'
-JSON_TTYPE = 'UXF^ttype'
+JSON_IMPORTS = 'UXF^imports'
+JSON_TCLASSES = 'UXF^ttypes'
+JSON_TCLASS = 'UXF^ttype'
 JSON_DATA = 'UXF^data'
 JSON_BYTES = 'UXF^bytes'
 JSON_DATE = 'UXF^date'
@@ -740,10 +784,11 @@ VTYPE = 'vtype'
 
 PREFIX = '''
 uxfconvert.py <infile.uxf[.gz]> <outfile.{csv,json,sqlite,xml}>
-uxfconvert.py [-i|--indent=N] [-f|--fieldnames] \
-<infile.{csv,ini,json,sqlite,xml}> <outfile.uxf[.gz]>
-uxfconvert.py [-i|--indent=N] [-f|--fieldnames] \
-<infile1.csv> [infile2.csv ... infileM.csv] <outfile.uxf[.gz]>
+uxfconvert.py [-d|--dropunused] [-r|--replaceimports] \
+[-f|--fieldnames] <infile.{csv,ini,json,sqlite,xml}> <outfile.uxf[.gz]>
+uxfconvert.py [-d|--dropunused] [-r|--replaceimports] \
+[-f|--fieldnames] <infile1.csv> [infile2.csv ... infileM.csv] \
+<outfile.uxf[.gz]>
 
 '''
 
@@ -753,6 +798,11 @@ If the outfile is .uxf.gz, the output will be gzip compressed UXF.
 
 Indent defaults to 2 (uxf's default); but can be set to any value 0-8.
 
+Use -d or --dropunused to drop unused ttype definitions and imports.
+
+Use -r or --replaceimports to replace imports with ttype definitions
+to make the outfile standalone (i.e., not dependent on any imports).
+
 If fieldnames is set and the infile(s) is(are) csv the first row of each
 infile will be read as field (column) names; otherwise all rows will be
 assumed to contain values.
@@ -785,8 +835,8 @@ whitespace differences in strings).
 
 Support for uxf to uxf conversions is provided by the uxf.py module itself,
 which can be run directly or via python, e.g., `uxf.py infile.uxf
-outfile.uxf` or `python3 -m uxf infile.uxf outfile.uxf` with the same indent
-and compression options as here, plus additional options (use uxf.py's -h or
+outfile.uxf` or `python3 -m uxf infile.uxf outfile.uxf` with the same
+compression option as here, plus additional options (use uxf.py's -h or
 --help for details).'''
 
 
